### Analysis run with R version 3.4.3 (2017-11-30) and platform x86_64-apple-darwin15.6.0 ###
### Running under: macOS  10.14.5 ###
### File to create descriptive statistics for Lutscher et al. (2019) ###
### At Home and Abroad: The Use of Denial-of-Service Attacks ###
### during Elections in Non-democratic Regimes ###

##Set working directory##
wd <- "" #Change to folder where R files and dosel_final.csv is stored
setwd(wd)
rm(list = ls())

#Load packages
library(data.table) #Version 1.12.0
library(stargazer) #Version  5.2.2
library(ggplot2) #Version 3.1.0
library(sp) #Version 1.3-1
library(RColorBrewer) #Version 1.1-2
library(cshapes) #Version 0.6
library(xtable) #Version 1.8-2

#Load dataset
dosel<-as.data.table(read.csv("dosel_final.csv"))
dosel$yearweek<-as.Date(dosel$yearweek)
dosel$X<-NULL

#v2x_polyarchy = inverted electoral democracy index by V-dem project

##### Table A.1: Summary statistics of main variables #####
summary<-dosel[,c("number_dos_unf","number_dos_outside","v2x_polyarchy","election_period_eg","year"),with=T]
summary$number_dos_unf <- log(summary$number_dos_unf+1)
summary$number_dos_outside <- log(summary$number_dos_outside+1)
summary$number_dos_outside <- ifelse(summary$year<2014,NA,summary$number_dos_outside)
summary$year <- NULL

stargazer(summary,
          covariate.labels = c("DoS attacks (ln)","DoS attacks on foreign hosts (ln)",
                                "Autocracy index","Election period"), 
          type="latex")

#####

#### Figure 1: Temporal development of DoS attacks (worldwide) ####

#Aggregated attacks to worldwide development
world<-aggregate(dosel$number_dos_unf,by=list(yearweek=dosel$yearweek),sum)
world$total<-world$x
world$x<-NULL

ggplot(world, aes(yearweek, total))+ geom_line() + xlab("Years") + ylab("Number of DoS attacks") +
              geom_smooth(color="black",linetype = "dashed") + theme_minimal() +
  scale_y_continuous(labels = scales::comma)

dev.print(pdf,"./Figures/world.pdf",width=8, height=4)


#### Figure 2: Spatial distribution of DoS attacks (worldwide) #####

#Aggregate attacks per country
country <- dosel[,sum(number_dos_unf,na.rm=T),by=list(ccode)]
country$COWCODE <- country$ccode
#Load map (using cshapes)
wrld_simpl<-cshp(as.Date("2016-1-1"))
wrld_simpl <- merge(wrld_simpl,country,by=c("COWCODE"),all.x=T)

breaks <- round(quantile(wrld_simpl$V1,seq(0,1,1/5),na.rm=T),0)
cols <- brewer.pal(length(breaks),"Greys")

plot(wrld_simpl,col=cols[findInterval(wrld_simpl$V1,breaks,all.inside =T)],
     density=ifelse(is.na(wrld_simpl$V1),30,NA))

legend <- c("0 - 164","165 - 1,454","1,455 - 8,432","8,433 - 54,720",
            "54,720 - 12,254,114", "No election (IFES)")

cols <- cbind(cols,"#000000")
legend(x=c("bottomleft"), # position
       legend = legend,
       title = "",
       fill =cols,
       density=c(NA, NA, NA, NA, NA, 30),
       cex = 0.8,
       bty = "n")# border

dev.print(pdf,"./Figures/Map.pdf",width=10, height=7)



#### Figure 3: Development around elections for some countries ####

# Iran 2009
p<-ggplot(dosel[Name=="Iran, Islamic Republic of" & yearweek>="2009-05-15" & yearweek<="2009-07-15"], aes(yearweek, number_dos_unf))+ 
  geom_step(col="black",lwd=1)  + xlab("Date") + ggtitle("Iran (2009)")+
  ylab("Number of DoS attacks on domestic hosts") + scale_x_date(date_breaks = "2 weeks", date_labels = " %d %b") +theme_minimal() +
  theme(plot.title = element_text(hjust = 0.5,face="bold"))
x<-p+geom_vline(data=dosel,aes(xintercept=as.numeric(as.Date("2009-06-12"))),color="black",lwd=1,lty="dashed")

# Turkey 2015 (outside)
p<-ggplot(dosel[Name=="Turkey" & yearweek>="2015-10-01" & yearweek<="2015-12-01"], aes(yearweek, number_dos_outside))+ 
  geom_step(col="black",lwd=1)  + xlab("Date") + ggtitle("Turkey (2015)")+
  ylab("Number of DoS attacks on foreign hosts") + scale_x_date(date_breaks = "2 weeks", date_labels =  "%d %b") +theme_minimal() +
  theme(plot.title = element_text(hjust = 0.5,face="bold"))

y<-p+geom_vline(data=dosel,aes(xintercept=as.numeric(as.Date("2015-11-01"))),color="black",lwd=1,lty="dashed")

#Gambia 2016
x1<-ggplot(dosel[Name=="Gambia" & yearweek>="2016-11-11" & yearweek<="2016-12-25"], aes(yearweek, number_dos_unf))+ 
  geom_step(col="black",lwd=1)  + xlab("Date") + ggtitle("Gambia (2016)")+
  ylab("Number of DoS attacks on domestic hosts") + scale_x_date(date_breaks = "1 weeks", date_labels =  "%d %b") +theme_minimal() +
  theme(plot.title = element_text(hjust = 0.5,face="bold"))

z1<-x1+geom_vline(data=dosel,aes(xintercept=as.numeric(as.Date("2016-12-01"))),color="black",lwd=1,lty="dashed")

#Gambia 2016 (outside)
x2<-ggplot(dosel[Name=="Gambia" & yearweek>="2016-11-11"  & yearweek<="2016-12-25"], aes(yearweek, number_dos_outside))+ 
  geom_step(col="black",lwd=1)  + xlab("Date") + ggtitle(" Gambia (2016)")+
  ylab("Number of DoS attacks on foreign hosts") + scale_x_date(date_breaks = "1 weeks", date_labels =  "%d %b") +theme_minimal() +
  theme(plot.title = element_text(hjust = 0.5,face="bold"))

z2<-x2+geom_vline(data=dosel,aes(xintercept=as.numeric(as.Date("2016-12-01"))),color="black",lwd=1,lty="dashed")

##### Set Multiplot function ####
# Multiple plot function
# Retrieved from http://www.cookbook-r.com/Graphs/Multiple_graphs_on_one_page_(ggplot2)/
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  library(grid)
  
  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)
  
  numPlots = length(plots)
  
  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                     ncol = cols, nrow = ceiling(numPlots/cols))
  }
  
  if (numPlots==1) {
    print(plots[[1]])
    
  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))
    
    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))
      
      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}
##### Plot Figure 3: Development around elections for some countries ####

multiplot(x,z1,y,z2,cols=2)
dev.print(pdf,"./Figures/examples.pdf",width=11, height=10)

##### Table 1: Comparison of absolute number of attacks #####
#Compare weeks with election and without & between types

#Attacks on country
election_demo<- dosel[v2x_polyarchy<0.5 & election_period_eg==1]$number_dos_unf
election_n_demo<- dosel[v2x_polyarchy<0.5 & election_period_eg!=1]$number_dos_unf


election_auto<-dosel[v2x_polyarchy>=0.5 & election_period_eg==1]$number_dos_unf
election_n_auto<-dosel[v2x_polyarchy>=0.5 & election_period_eg!=1]$number_dos_unf


#Attacks on foreign hosts
election_demo_o<-dosel[v2x_polyarchy<0.5 & election_period_eg==1 & year>=2014]$number_dos_outside
election_n_demo_o<-dosel[v2x_polyarchy<0.5 & election_period_eg!=1 & year>=2014]$number_dos_outside



election_aut_o<-dosel[v2x_polyarchy>=0.5 & election_period_eg==1 & year>=2014]$number_dos_outside
election_n_aut_o<-dosel[v2x_polyarchy>=0.5 & election_period_eg!=1 & year>=2014]$number_dos_outside



tab <- matrix(c(mean(election_demo,na.rm=T),mean(election_auto,na.rm=T),
                mean(election_demo_o,na.rm=T),mean(election_aut_o,na.rm=T),
                mean(election_n_demo,na.rm=T),mean(election_n_auto,na.rm=T),
                mean(election_n_demo_o,na.rm=T),mean(election_n_aut_o,na.rm=T)
                ),ncol=4,byrow=TRUE)
colnames(tab) <- c("DoS attacks on country (democracy)",
                   "DoS attacks on country (autocracy)",
                   "DoS attacks on foreign hosts (democracy)",
                   "DoS attacks on foreign hosts(autocracy)")
rownames(tab) <- c("Elections","No elections")
tab <- round(tab,2)
tab <- as.table(tab)
xtable(tab,digits=2)